Package: sodium
Type: Package
Title: A Modern and Easy-to-Use Crypto Library
Version: 1.4.0
Authors@R: person("Jeroen", "Ooms", role = c("aut", "cre"), email = "jeroenooms@gmail.com",
    comment = c(ORCID = "0000-0002-4035-0289"))
Description: Bindings to 'libsodium' <https://doc.libsodium.org/>: a modern, 
    easy-to-use software library for encryption, decryption, signatures, password
    hashing and more. Sodium uses curve25519, a state-of-the-art Diffie-Hellman 
    function by Daniel Bernstein, which has become very popular after it was 
    discovered that the NSA had backdoored Dual EC DRBG.
License: MIT + file LICENSE
URL: https://docs.ropensci.org/sodium/ https://github.com/r-lib/sodium
BugReports: https://github.com/r-lib/sodium/issues
SystemRequirements: libsodium (>= 1.0.3)
VignetteBuilder: knitr
Suggests: knitr, rmarkdown
RoxygenNote: 7.2.3
NeedsCompilation: yes
Packaged: 2024-12-16 14:16:42 UTC; jeroen
Author: Jeroen Ooms [aut, cre] (<https://orcid.org/0000-0002-4035-0289>)
Maintainer: Jeroen Ooms <jeroenooms@gmail.com>
Repository: CRAN
Date/Publication: 2024-12-16 15:00:05 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-26 11:16:32 UTC; unix
Archs: sodium.so.dSYM

# Generated by roxygen2: do not edit by hand

export(argon2)
export(auth_decrypt)
export(auth_encrypt)
export(bin2hex)
export(chacha20)
export(data_decrypt)
export(data_encrypt)
export(data_tag)
export(diffie_hellman)
export(hash)
export(hex2bin)
export(keygen)
export(password_store)
export(password_verify)
export(pubkey)
export(random)
export(salsa20)
export(scrypt)
export(sha256)
export(sha512)
export(shorthash)
export(sig_keygen)
export(sig_pubkey)
export(sig_sign)
export(sig_verify)
export(simple_decrypt)
export(simple_encrypt)
export(xchacha20)
export(xsalsa20)
useDynLib(sodium,R_auth_sha256)
useDynLib(sodium,R_auth_sha512)
useDynLib(sodium,R_crypto_generichash)
useDynLib(sodium,R_crypto_secret_auth)
useDynLib(sodium,R_crypto_secret_decrypt)
useDynLib(sodium,R_crypto_secret_encrypt)
useDynLib(sodium,R_crypto_secret_verify)
useDynLib(sodium,R_crypto_shorthash)
useDynLib(sodium,R_diffie_hellman)
useDynLib(sodium,R_keygen)
useDynLib(sodium,R_password_hash)
useDynLib(sodium,R_password_verify)
useDynLib(sodium,R_pubkey)
useDynLib(sodium,R_pwhash)
useDynLib(sodium,R_pwhash_argon2)
useDynLib(sodium,R_randombytes_buf)
useDynLib(sodium,R_seal_box)
useDynLib(sodium,R_seal_open)
useDynLib(sodium,R_secure_recv)
useDynLib(sodium,R_secure_send)
useDynLib(sodium,R_sha256)
useDynLib(sodium,R_sha512)
useDynLib(sodium,R_sig_keygen)
useDynLib(sodium,R_sig_pubkey)
useDynLib(sodium,R_sig_sign)
useDynLib(sodium,R_sig_verify)
useDynLib(sodium,R_sodium_bin2hex)
useDynLib(sodium,R_sodium_hex2bin)
useDynLib(sodium,R_stream_chacha20)
useDynLib(sodium,R_stream_salsa20)
useDynLib(sodium,R_stream_xchacha20)
useDynLib(sodium,R_stream_xsalsa20)
useDynLib(sodium,R_xor)

## ----echo = FALSE, message = FALSE--------------------------------------------
knitr::opts_chunk$set(comment = "")
library(sodium)

## -----------------------------------------------------------------------------
test <- hash(charToRaw("test 123"))
str <- bin2hex(test)
print(str)
hex2bin(str)

## -----------------------------------------------------------------------------
secret <- random(8)
print(secret)

## -----------------------------------------------------------------------------
# Generate keys from passphrase
passphrase <- charToRaw("This is super secret")
hash(passphrase)
hash(passphrase, size = 16)
hash(passphrase, size = 64)

## -----------------------------------------------------------------------------
key <- hash(charToRaw("This is a secret passphrase"))
msg <- serialize(iris, NULL)

# Encrypt with a random nonce
nonce <- random(24)
cipher <- data_encrypt(msg, key, nonce)

# Decrypt with same key and nonce
orig <- data_decrypt(cipher, key, nonce)
identical(iris, unserialize(orig))

## -----------------------------------------------------------------------------
key <- hash(charToRaw("This is a secret passphrase"))
msg <- serialize(iris, NULL)
mytag <- data_tag(msg, key)

## -----------------------------------------------------------------------------
stopifnot(identical(mytag, data_tag(msg, key)))

## -----------------------------------------------------------------------------
key <- keygen()
pub <- pubkey(key)

## -----------------------------------------------------------------------------
# Encrypt message with pubkey
msg <- serialize(iris, NULL)
ciphertext <- simple_encrypt(msg, pub)

# Decrypt message with private key
out <- simple_decrypt(ciphertext, key)
stopifnot(identical(out, msg))

## -----------------------------------------------------------------------------
# Generate signature keypair
key <- sig_keygen()
pubkey <- sig_pubkey(key)

# Create signature with private key
msg <- serialize(iris, NULL)
sig <- sig_sign(msg, key)
print(sig)

# Verify a signature from public key
sig_verify(msg, sig, pubkey)

## -----------------------------------------------------------------------------
# Bob's keypair:
bob_key <- keygen()
bob_pubkey <- pubkey(bob_key)

# Alice's keypair:
alice_key <- keygen()
alice_pubkey <- pubkey(alice_key)

# Bob sends encrypted message for Alice:
msg <- charToRaw("TTIP is evil")
ciphertext <- auth_encrypt(msg, bob_key, alice_pubkey)

# Alice verifies and decrypts with her key
out <- auth_decrypt(ciphertext, alice_key, bob_pubkey)
stopifnot(identical(out, msg))

# Alice sends encrypted message for Bob
msg <- charToRaw("Let's protest")
ciphertext <- auth_encrypt(msg, alice_key, bob_pubkey)

# Bob verifies and decrypts with his key
out <- auth_decrypt(ciphertext, bob_key, alice_pubkey)
stopifnot(identical(out, msg))

## ----echo = FALSE, message = FALSE--------------------------------------------
knitr::opts_chunk$set(comment = "")
library(sodium)

# hack for printable bits 
random <- function(n = 1){
  if(n != nchar("TTIP is evil"))
    return(sodium::random(n))
  repeat {
    x <- sodium::random(n)
    y <- base::xor(charToRaw("TTIP is evil"), x)
    if(all(c(x,y) != 0)) return(x) 
  }
}

## -----------------------------------------------------------------------------
# XOR two (8bit) bytes 'x' and 'y'
x <- as.raw(0x7a)
y <- as.raw(0xe4)
z <- base::xor(x, y)
dput(z)

# Show the bits in each byte
cbind(x = rawToBits(x), y = rawToBits(y), z = rawToBits(z))

## -----------------------------------------------------------------------------
# Encrypt message using random one-time-pad
msg <- charToRaw("TTIP is evil")
one_time_pad <- random(length(msg))
ciphertext <- base::xor(msg, one_time_pad)

# It's really encrypted
rawToChar(ciphertext)

# Decrypt with same pad
rawToChar(base::xor(ciphertext, one_time_pad))

## -----------------------------------------------------------------------------
password <- "My secret passphrase"
key <- hash(charToRaw(password))
nonce <- random(8)
chacha20(size = 20, key, nonce)

## -----------------------------------------------------------------------------
salsa20(size = 20, key, nonce)

## -----------------------------------------------------------------------------
# Illustrative example.
sha256_ctr <- function(size, key, nonce){
  n <- ceiling(size/32)
  output <- raw()
  for(i in 1:n){
    counter <- packBits(intToBits(i))
    block <- sha256(c(key, nonce, counter))
    output <- c(output, block)
  }
  return(output[1:size])
}

## -----------------------------------------------------------------------------
password <- "My secret passphrase"
key <- hash(charToRaw(password))
nonce <- random(8)
sha256_ctr(50, key, nonce)

## -----------------------------------------------------------------------------
# Encrypt 'message' using 'password'
myfile <- file.path(R.home(), "COPYING")
message <- readBin(myfile, raw(), file.info(myfile)$size)
passwd <- charToRaw("My secret passphrase")

## -----------------------------------------------------------------------------
# Basic secret key encryption
key <- hash(passwd)
nonce8 <- random(8)
stream <- chacha20(length(message), key, nonce8)
ciphertext <- base::xor(stream, message)

## -----------------------------------------------------------------------------
# Decrypt with the same key
key <- hash(charToRaw("My secret passphrase"))
stream <- chacha20(length(ciphertext), key, nonce8)
out <- base::xor(ciphertext, stream)

# Print part of the message
cat(substring(rawToChar(out), 1, 120))

## -----------------------------------------------------------------------------
# Create keypair
key <- keygen()
pub <- pubkey(key)

# Encrypt message for receiver using his/her public key
msg <- serialize(iris, NULL)
ciphertext <- simple_encrypt(msg, pub)

# Receiver decrypts with his/her private key
out <- simple_decrypt(ciphertext, key)
identical(msg, out)

## -----------------------------------------------------------------------------
# Bob generates keypair
bob_key <- keygen()
bob_pubkey <- pubkey(bob_key)

# Alice generates keypair
alice_key <- keygen()
alice_pubkey <- pubkey(alice_key)

# After Bob and Alice exchange pubkey they can both derive the secret
alice_secret <- diffie_hellman(alice_key, bob_pubkey)
bob_secret <- diffie_hellman(bob_key, alice_pubkey)
identical(alice_secret, bob_secret)

#' Keypair Generation
#'
#' Functions to generate a random private key and calculate the corresponding curve25519
#' public key.
#'
#' Asymmetric methods rely on public-private keypairs. The private keys are secret and
#' should never be shared with anyone. The public key on the other hand is not confidential
#' and should be shared with the other parties. Public keys are typically published on the
#' users's website or posted in public directories or keyservers.
#'
#' The two main applications for public key cryptography are encryption and authentication.
#'
#' In public key encryption, data that is encrypted using a public key can only be
#' decrypted using the corresponding private key. This allows anyone to send somebody a
#' secure message by encrypting it with the receivers public key. The encrypted message
#' will only be readable by the owner of the corresponding private key. Basic encryption
#' is implemented in \link{simple_encrypt}.
#'
#' Authentication works the other way around. In public key authentication, the owner of the
#' private key creates a 'signature' (an authenticated checksum) for a message in a way that
#' allows anyone who knows the user's public key to verify that this message was indeed signed
#' by the owner of the private key.
#'
#' If both sender and receiver know each other's public key, the two methods can be combined
#' so that each message going back and forth is signed by the sender and encrypted for the
#' receiver. This protects both against eavesdropping and MITM tampering, creating a fully
#' secure channel.
#'
#' @export
#' @rdname keygen
#' @name Key generation
#' @param key private key for which to calculate the public key
#' @param seed random data to seed the keygen
#' @useDynLib sodium R_keygen
#' @examples # Create keypair
#' key <- keygen()
#' pub <- pubkey(key)
#'
#' # Basic encryption
#' msg <- serialize(iris, NULL)
#' ciphertext <- simple_encrypt(msg, pub)
#' out <- simple_decrypt(ciphertext, key)
#' stopifnot(identical(msg, out))
keygen <- function(seed = random(32)){
  stopifnot(is.raw(seed))
  .Call(R_keygen, seed)
}

#' @export
#' @rdname keygen
#' @useDynLib sodium R_pubkey
pubkey <- function(key){
  stopifnot(is.raw(key))
  .Call(R_pubkey, key)
}

#' Stream Ciphers
#'
#' Generate deterministic streams of random data based off a secret key and random nonce.
#'
#' You usually don't need to call these methods directly. For local encryption
#' use \link{data_encrypt}. For secure communication use \link{simple_encrypt} or
#' \link{auth_encrypt}.
#'
#' Random streams form the basis for most cryptographic methods. Based a shared secret
#' (the key) we generate a predictable random data stream of equal length as the message
#' we need to encrypt. Then we \link{xor} the message data with this random stream,
#' which effectively inverts each byte in the message with probabiliy 0.5. The message
#' can be decrypted by re-generating exactly the same random data stream and \link{xor}'ing
#' it back. See the examples.
#'
#' Each stream generator requires a \code{key} and a \code{nonce}. Both are required to re-generate
#' the same stream for decryption. The key forms the shared secret and should only known to
#' the trusted parties. The \code{nonce} is not secret and should be stored or sent along
#' with the ciphertext. The purpose of the \code{nonce} is to make a random stream unique
#' to protect gainst re-use attacks. This way you can re-use a your key to encrypt multiple
#' messages, as long as you never re-use the same nonce.
#'
#' @export
#' @useDynLib sodium R_stream_chacha20
#' @rdname stream
#' @aliases stream
#' @name Stream ciphers
#' @param size length of cipher stream in bytes
#' @param key secret key used by the cipher
#' @param nonce non-secret unique data to randomize the cipher
#' @references \url{https://libsodium.gitbook.io/doc/advanced/stream_ciphers/xsalsa20}
#' @examples # Very basic encryption
#' myfile <- file.path(R.home(), "COPYING")
#' message <- readBin(myfile, raw(), file.info(myfile)$size)
#' passwd <- charToRaw("My secret passphrase")
#'
#' # Encrypt:
#' key <- hash(passwd)
#' nonce8 <- random(8)
#' stream <- chacha20(length(message), key, nonce8)
#' ciphertext <- base::xor(stream, message)
#'
#' # Decrypt:
#' stream <- chacha20(length(ciphertext), key, nonce8)
#' out <- base::xor(ciphertext, stream)
#' stopifnot(identical(out, message))
#'
#' # Other stream ciphers
#' stream <- salsa20(10000, key, nonce8)
#' stream <- xsalsa20(10000, key, random(24))
#' stream <- xchacha20(10000, key, random(24))
#'
chacha20 <- function(size, key, nonce){
  stopifnot(is.numeric(size))
  stopifnot(is.raw(key))
  stopifnot(is.raw(nonce))
  .Call(R_stream_chacha20, size, key, nonce)
}

#' @export
#' @useDynLib sodium R_stream_xchacha20
#' @rdname stream
xchacha20 <- function(size, key, nonce){
  stopifnot(is.numeric(size))
  stopifnot(is.raw(key))
  stopifnot(is.raw(nonce))
  .Call(R_stream_xchacha20, size, key, nonce)
}

#' @export
#' @useDynLib sodium R_stream_salsa20
#' @rdname stream
salsa20 <- function(size, key, nonce){
  stopifnot(is.numeric(size))
  stopifnot(is.raw(key))
  stopifnot(is.raw(nonce))
  .Call(R_stream_salsa20, size, key, nonce)
}

#' @export
#' @useDynLib sodium R_stream_xsalsa20
#' @rdname stream
xsalsa20 <- function(size, key, nonce){
  stopifnot(is.numeric(size))
  stopifnot(is.raw(key))
  stopifnot(is.raw(nonce))
  .Call(R_stream_xsalsa20, size, key, nonce)
}

#' Hash Functions
#'
#' Functions to calculate cryptographic hash of a message, with optionally a key for
#' HMAC applications. For storing passwords, use \link{password_store} instead.
#'
#' The generic \code{hash} function is recommended for most applications. It uses
#' dynamic length
#' \href{https://libsodium.gitbook.io/doc/hashing/generic_hashing}{BLAKE2b}
#' where output size can be any value between 16 bytes (128bit) and 64 bytes (512bit).
#'
#' The \link{scrypt} hash function is designed to be CPU and memory expensive to protect
#' against brute force attacks. This algorithm is also used by the \link{password_store}
#' function.
#'
#' The \link{argon2} hash function is also designed to be CPU and memory expensive to protect
#' against brute force attacks. Argon2 is a password-hashing function that summarizes the
#' state of the art in the design of memory-hard functions
#'
#' The \code{shorthash} function is a special 8 byte (64 bit) hash based on
#' \href{https://libsodium.gitbook.io/doc/hashing/short-input_hashing}{SipHash-2-4}.
#' The output of this function is only 64 bits (8 bytes). It is useful for in e.g.
#' Hash tables, but it should not be considered collision-resistant.
#'
#' Hash functions can be used for HMAC by specifying a secret \code{key}. They key size
#' for \code{shorthash} is 16 bytes, for \code{sha256} it is 32 bytes and for \code{sha512}
#' it is 64 bytes. For \code{hash} the key size can be any value between 16 and 62,
#' recommended is at least 32.
#'
#' @rdname hash
#' @name Hash functions
#' @aliases hashing
#' @references \url{https://libsodium.gitbook.io/doc/hashing/generic_hashing}
#' @useDynLib sodium R_crypto_generichash
#' @param buf data to be hashed
#' @param key key for HMAC hashing. Optional, except for in \code{shorthash}.
#' @export
#' @examples # Basic hashing
#' msg <- serialize(iris, NULL)
#' hash(msg)
#' sha256(msg)
#' sha512(msg)
#' scrypt(msg)
#'
#' # Generate keys from passphrase
#' passphrase <- charToRaw("This is super secret")
#' key <- hash(passphrase)
#' shortkey <- hash(passphrase, size = 16)
#' longkey <- hash(passphrase, size = 64)
#'
#' # HMAC (hashing with key)
#' hash(msg, key = key)
#' shorthash(msg, shortkey)
#' sha256(msg, key = key)
#' sha512(msg, key = longkey)
hash <- function(buf, key = NULL, size = 32){
  stopifnot(is.raw(buf))
  stopifnot(is.null(key) || is.raw(key))
  .Call(R_crypto_generichash, buf, size, key)
}

#' @export
#' @rdname hash
#' @param salt non-confidential random data to seed the algorithm
#' @useDynLib sodium R_pwhash
scrypt <- function(buf, salt = raw(32), size = 32){
  stopifnot(is.raw(buf))
  stopifnot(is.raw(salt))
  stopifnot(is.numeric(size))
  .Call(R_pwhash, buf, salt, size)
}

#' @export
#' @rdname hash
#' @useDynLib sodium R_pwhash_argon2
argon2 <- function(buf, salt = raw(16), size = 32){
  stopifnot(is.raw(buf))
  stopifnot(is.raw(salt))
  stopifnot(is.numeric(size))
  .Call(R_pwhash_argon2, buf, salt, size)
}

#' @export
#' @rdname hash
#' @useDynLib sodium R_crypto_shorthash
shorthash <- function(buf, key){
  stopifnot(is.raw(buf))
  stopifnot(is.raw(key))
  .Call(R_crypto_shorthash, buf, key)
}

#' @rdname hash
#' @useDynLib sodium R_sha512 R_auth_sha512
#' @export
sha512 <- function(buf, key = NULL){
  stopifnot(is.raw(buf))
  if(length(key)){
    stopifnot(is.raw(key))
    .Call(R_auth_sha512, buf, key)
  } else {
    .Call(R_sha512, buf)
  }
}

#' @rdname hash
#' @param size length of the output hash. Must be between 16 and 64 (recommended is 32)
#' @useDynLib sodium R_sha256 R_auth_sha256
#' @export
sha256 <- function(buf, key = NULL){
  stopifnot(is.raw(buf))
  if(length(key)){
    stopifnot(is.raw(key))
    .Call(R_auth_sha256, buf, key)
  } else {
    .Call(R_sha256, buf)
  }
}

#' Create and Verify Signatures
#'
#' Cryptographic signatures can be used to verify the integrity of a message using
#' the author's public key.
#'
#' A signature is an authenticated checksum that can be used to check that a message
#' (any data) was created by a particular author and was not tampered with. The signature is
#' created using a private key and can be verified from the corresponding public key.
#'
#' Signatures are used when the message itself is not confidential but integrity is
#' important. A common use is for software repositories where maintainers include
#' a signature of the package index. This allows client package managers to verify
#' that the binaries were not modified by intermediate parties in the distribution
#' process.
#'
#' For confidential data, use authenticated encryption (\link{auth_encrypt})
#' which allows for sending signed and encrypted messages in a single method.
#'
#' Currently sodium requires a different type of key pairfor signatures  (ed25519)
#' than for encryption (curve25519).
#'
#' @rdname sig
#' @name Signatures
#' @aliases sig
#' @export
#' @useDynLib sodium R_sig_sign
#' @param msg message to sign
#' @param key private key to sign message with
#' @param sig a signature generated by \code{signature_sign}
#' @param pubkey a public key of the keypair used by the signature
#' @references \url{https://doc.libsodium.org/public-key_cryptography/public-key_signatures.html}
#' @examples # Generate keypair
#' key <- sig_keygen()
#' pubkey <- sig_pubkey(key)
#'
#' # Create signature
#' msg <- serialize(iris, NULL)
#' sig <- sig_sign(msg, key)
#' sig_verify(msg, sig, pubkey)
sig_sign <- function(msg, key){
  stopifnot(is.raw(msg))
  stopifnot(is.raw(key))
  .Call(R_sig_sign, msg, key)
}

#' @export
#' @rdname sig
#' @useDynLib sodium R_sig_verify
sig_verify <- function(msg, sig, pubkey){
  stopifnot(is.raw(msg))
  stopifnot(is.raw(sig))
  stopifnot(is.raw(pubkey))
  .Call(R_sig_verify, msg, sig, pubkey)
}

#' @export
#' @rdname sig
#' @useDynLib sodium R_sig_keygen
#' @param seed random data to seed the keygen
sig_keygen <- function(seed = random(32)){
  stopifnot(is.raw(seed))
  .Call(R_sig_keygen, seed)
}

#' @export
#' @rdname sig
#' @useDynLib sodium R_sig_pubkey
sig_pubkey <- function(key){
  stopifnot(is.raw(key))
  .Call(R_sig_pubkey, key)
}

## ---- echo = FALSE, message = FALSE--------------------------------------
knitr::opts_chunk$set(comment = "")
library(sodium)

# hack for printable bits 
random <- function(n = 1){
  if(n != nchar("TTIP is evil"))
    return(sodium::random(n))
  repeat {
    x <- sodium::random(n)
    y <- base::xor(charToRaw("TTIP is evil"), x)
    if(all(c(x,y) != 0)) return(x) 
  }
}

## ------------------------------------------------------------------------
# XOR two (8bit) bytes 'x' and 'y'
x <- as.raw(0x7a)
y <- as.raw(0xe4)
z <- base::xor(x, y)
dput(z)

# Show the bits in each byte
cbind(x = rawToBits(x), y = rawToBits(y), z = rawToBits(z))

## ------------------------------------------------------------------------
# Encrypt message using random one-time-pad
msg <- charToRaw("TTIP is evil")
one_time_pad <- random(length(msg))
ciphertext <- base::xor(msg, one_time_pad)

# It's really encrypted
rawToChar(ciphertext)

# Decrypt with same pad
rawToChar(base::xor(ciphertext, one_time_pad))

## ------------------------------------------------------------------------
password <- "My secret passphrase"
key <- hash(charToRaw(password))
nonce <- random(8)
chacha20(size = 20, key, nonce)

## ------------------------------------------------------------------------
salsa20(size = 20, key, nonce)

## ------------------------------------------------------------------------
# Illustrative example.
sha256_ctr <- function(size, key, nonce){
  n <- ceiling(size/32)
  output <- raw()
  for(i in 1:n){
    counter <- packBits(intToBits(i))
    block <- sha256(c(key, nonce, counter))
    output <- c(output, block)
  }
  return(output[1:size])
}

## ------------------------------------------------------------------------
password <- "My secret passphrase"
key <- hash(charToRaw(password))
nonce <- random(8)
sha256_ctr(50, key, nonce)

## ------------------------------------------------------------------------
# Encrypt 'message' using 'password'
myfile <- file.path(R.home(), "COPYING")
message <- readBin(myfile, raw(), file.info(myfile)$size)
passwd <- charToRaw("My secret passphrase")

## ------------------------------------------------------------------------
# Basic secret key encryption
key <- hash(passwd)
nonce8 <- random(8)
stream <- chacha20(length(message), key, nonce8)
ciphertext <- base::xor(stream, message)

## ------------------------------------------------------------------------
# Decrypt with the same key
key <- hash(charToRaw("My secret passphrase"))
stream <- chacha20(length(ciphertext), key, nonce8)
out <- base::xor(ciphertext, stream)

# Print part of the message
cat(substring(rawToChar(out), 1, 120))

## ------------------------------------------------------------------------
# Create keypair
key <- keygen()
pub <- pubkey(key)

# Encrypt message for receiver using his/her public key
msg <- serialize(iris, NULL)
ciphertext <- simple_encrypt(msg, pub)

# Receiver decrypts with his/her private key
out <- simple_decrypt(ciphertext, key)
identical(msg, out)

## ------------------------------------------------------------------------
# Bob generates keypair
bob_key <- keygen()
bob_pubkey <- pubkey(bob_key)

# Alice generates keypair
alice_key <- keygen()
alice_pubkey <- pubkey(alice_key)

# After Bob and Alice exchange pubkey they can both derive the secret
alice_secret <- diffie_hellman(alice_key, bob_pubkey)
bob_secret <- diffie_hellman(bob_key, alice_pubkey)
identical(alice_secret, bob_secret)

#' Authenticated Encryption
#'
#' Exchange secure messages through curve25519 authenticated encryption.
#'
#' Authenticated encryption implements best practices for secure messaging.
#' It requires that both sender and receiver have a keypair and know each
#' other's public key. Each message gets authenticated with the key of the
#' sender and encrypted with the key of the receiver.
#'
#' Even though public keys are not confidential, you should not exchange them
#' over the same insecure channel you are trying to protect. If the connection
#' is being tampered with, the attacker could simply replace the key with another
#' one to hijack the interaction.
#'
#' Most people share their public key by posting them on their website or on a
#' public keyserver. Another alternative is having your public key signed by a
#' mutually trusted third party. HTTPS does this using Certificate Authorities.
#'
#' @export
#' @useDynLib sodium R_secure_send
#' @rdname messaging
#' @name Authenticated encryption
#' @references \url{https://libsodium.gitbook.io/doc/public-key_cryptography/authenticated_encryption}
#' @examples # Bob's keypair:
#' bob_key <- keygen()
#' bob_pubkey <- pubkey(bob_key)
#'
#' # Alice's keypair:
#' alice_key <- keygen()
#' alice_pubkey <- pubkey(alice_key)
#'
#' # Bob sends encrypted message for Alice:
#' msg <- charToRaw("TTIP is evil")
#' ciphertext <- auth_encrypt(msg, bob_key, alice_pubkey)
#'
#' # Alice verifies and decrypts with her key
#' out <- auth_decrypt(ciphertext, alice_key, bob_pubkey)
#' stopifnot(identical(out, msg))
#'
#' # Alice sends encrypted message for Bob
#' msg <- charToRaw("Let's protest")
#' ciphertext <- auth_encrypt(msg, alice_key, bob_pubkey)
#'
#' # Bob verifies and decrypts with his key
#' out <- auth_decrypt(ciphertext, bob_key, alice_pubkey)
#' stopifnot(identical(out, msg))
#' @param msg message to be encrypted
#' @param bin encrypted ciphertext generated by \code{secure_send}
#' @param key your own private key
#' @param pubkey other person's public key
#' @param nonce non-secret unique data to randomize the cipher
auth_encrypt <- function(msg, key, pubkey, nonce = random(24)){
  stopifnot(is.raw(msg))
  stopifnot(is.raw(key))
  stopifnot(is.raw(pubkey))
  stopifnot(is.raw(nonce))
  res <- .Call(R_secure_send, msg, key, pubkey, nonce)
  structure(res, nonce = nonce)
}

#' @export
#' @rdname messaging
#' @useDynLib sodium R_secure_recv
auth_decrypt <- function(bin, key, pubkey, nonce = attr(bin, "nonce")){
  stopifnot(is.raw(bin))
  stopifnot(is.raw(key))
  stopifnot(is.raw(pubkey))
  stopifnot(is.raw(nonce))
  .Call(R_secure_recv, bin, key, pubkey, nonce)
}

#' Sodium Utilities
#'
#' The functions \code{bin2hex} and \code{hex2bin} convert between binary (raw)
#' vectors and corresponding string in hexadecimal notation. The \code{random}
#' function generates \code{n} crypto secure random bytes.
#'
#' @useDynLib sodium R_sodium_bin2hex
#' @export
#' @rdname helpers
#' @name Sodium utilities
#' @aliases helpers
#' @family sodium
#' @param bin raw vector with binary data to convert to hex string
#' @examples # Convert raw to hex string and back
#' test <- charToRaw("test 123")
#' x <- bin2hex(test)
#' y <- hex2bin(x)
#' stopifnot(identical(test, y))
#' stopifnot(identical(x, paste(test, collapse = "")))
#'
#' # Parse text with characters
#' x2 <- paste(test, collapse = ":")
#' y2 <- hex2bin(x2, ignore = ":")
#' stopifnot(identical(test, y2))
bin2hex <- function(bin){
  stopifnot(is.raw(bin))
  .Call(R_sodium_bin2hex, bin)
}

#' @useDynLib sodium R_sodium_hex2bin
#' @export
#' @rdname helpers
#' @param hex a string with hexadecimal characters to parse into a binary (raw) vector.
#' @param ignore a string with characters to ignore from \code{hex}. See example.
hex2bin <- function(hex, ignore = ":"){
  stopifnot(is.character(hex))
  stopifnot(length(hex) == 1)
  stopifnot(is.character(ignore))
  stopifnot(length(ignore) == 1)
  .Call(R_sodium_hex2bin, hex, ignore)
}

#' @useDynLib sodium R_randombytes_buf
#' @export
#' @rdname helpers
#' @param n number of random bytes or numbers to generate
random <- function(n = 1){
  stopifnot(is.numeric(n))
  .Call(R_randombytes_buf, as.integer(n))
}

#' @useDynLib sodium R_xor
xor <- function(x, y){
  stopifnot(is.raw(x))
  stopifnot(is.raw(y))
  .Call(R_xor, x, y)
}

#' Symmetric Encryption and Tagging
#'
#' Encryption with authentication using a 256 bit shared secret. Mainly useful for
#' encrypting local data. For secure communication use public-key encryption
#' (\link{simple_encrypt} and \link{auth_encrypt}).
#'
#' Symmetric encryption uses a secret key to encode and decode a message. This can be
#' used to encrypt local data on disk, or as a building block for more complex methods.
#'
#' Because the same \code{secret} is used for both encryption and decryption, symmetric
#' encryption by itself is impractical for communication. For exchanging secure messages
#' with other parties, use assymetric (public-key) methods (see \link{simple_encrypt} or
#' \link{auth_encrypt}).
#'
#' The \code{nonce} is not confidential but required for decryption, and should be
#' stored or sent along with the ciphertext. The purpose of the \code{nonce} is to
#' randomize the cipher to protect gainst re-use attacks. This way you can use one
#' and the same secret for encrypting multiple messages.
#'
#' The \link{data_tag} function generates an authenticated hash that can be stored
#' alongside the data to be able to verify the integrity of the data later on. For
#' public key signatures see \code{sig_sign} instead.
#'
#' @export
#' @rdname symmetric
#' @name Symmetric encryption
#' @useDynLib sodium R_crypto_secret_encrypt
#' @param msg message to be encrypted
#' @param key shared secret key used for both encryption and decryption
#' @param bin encrypted ciphertext
#' @param nonce non-secret unique data to randomize the cipher
#' @references \url{https://libsodium.gitbook.io/doc/public-key_cryptography/authenticated_encryption}
#' @examples # 256-bit key
#' key <- sha256(charToRaw("This is a secret passphrase"))
#' msg <- serialize(iris, NULL)
#'
#' # Encrypts with random nonce
#' cipher <- data_encrypt(msg, key)
#' orig <- data_decrypt(cipher, key)
#' stopifnot(identical(msg, orig))
#'
#' # Tag the message with your key (HMAC)
#' tag <- data_tag(msg, key)
data_encrypt <- function(msg, key, nonce = random(24)){
  stopifnot(is.raw(msg))
  stopifnot(is.raw(key))
  out <- .Call(R_crypto_secret_encrypt, msg, key, nonce)
  structure(out, nonce = nonce)
}

#' @export
#' @rdname symmetric
#' @useDynLib sodium R_crypto_secret_decrypt
data_decrypt <- function(bin, key, nonce = attr(bin, "nonce")){
  stopifnot(is.raw(bin))
  stopifnot(is.raw(key))
  .Call(R_crypto_secret_decrypt, bin, key, nonce)
}

#' @export
#' @rdname symmetric
#' @useDynLib sodium R_crypto_secret_auth
data_tag <- function(msg, key){
  stopifnot(is.raw(msg))
  stopifnot(is.raw(key))
  .Call(R_crypto_secret_auth, msg, key)
}

#' @useDynLib sodium R_crypto_secret_verify
data_verify <- function(msg, key, tag){
  stopifnot(is.raw(msg))
  stopifnot(is.raw(tag))
  stopifnot(is.raw(key))
  .Call(R_crypto_secret_verify, msg, key, tag)
}

#' Diffie-Hellman
#'
#' The Diffie-Hellman key exchange method allows two parties that have no prior knowledge
#' of each other to jointly establish a shared secret key over an insecure channel. This
#' key can then be used to encrypt subsequent communications using a symmetric key cipher.
#'
#' Encryption methods as implemented in \link{data_encrypt} require that parties have a
#' shared secret key. But often we wish to establish a secure channel with a party we have
#' no prior relationship with. Diffie-hellman is a method for jointly agreeing on a shared
#' secret without ever exchanging the secret itself. Sodium implements
#' \href{https://en.wikipedia.org/wiki/Curve25519}{Curve25519}, a state-of-the-art Diffie-Hellman
#' function suitable for a wide variety of applications.
#'
#' The method conists of two steps (see examples). First, both parties generate a random private
#' key and derive the corresponding public key using \link{pubkey}. These public keys are not
#' confidential and can be exchanged over an insecure channel. After the public keys are exchanged,
#' both parties will be able to calculate the (same) shared secret by combining his/her own private
#' key with the other person's public key using \link{diffie_hellman}.
#'
#' After the shared secret has been established, the private and public keys are disposed,
#' and parties can start encrypting communications based on the shared secret using e.g.
#' \link{data_encrypt}. Because the shared secret cannot be calculated using only the public
#' keys, the process is safe from eavesdroppers.
#'
#' @export
#' @rdname diffie
#' @name Diffie-Hellman
#' @aliases diffie
#' @useDynLib sodium R_diffie_hellman
#' @references \url{https://doc.libsodium.org/advanced/scalar_multiplication.html}
#' @param key your private key
#' @param pubkey other person's public key
#' @return Returns a shared secret key which can be used in e.g. \link{data_encrypt}.
#' @examples # Bob generates keypair
#' bob_key <- keygen()
#' bob_pubkey <- pubkey(bob_key)
#'
#' # Alice generates keypair
#' alice_key <- keygen()
#' alice_pubkey <- pubkey(alice_key)
#'
#' # After Bob and Alice exchange pubkey they can both derive the secret
#' alice_secret <- diffie_hellman(alice_key, bob_pubkey)
#' bob_secret <- diffie_hellman(bob_key, alice_pubkey)
#' stopifnot(identical(alice_secret, bob_secret))
diffie_hellman <- function(key, pubkey){
  stopifnot(is.raw(key))
  stopifnot(is.raw(pubkey))
  .Call(R_diffie_hellman, key, pubkey)
}

#' Anonymous Public-key Encryption (Sealed Box)
#'
#' Create an encrypted message (sealed box) from a curve25519 public key.
#'
#' Simple public key encryption allows for sending anonymous encrypted messages to
#' a recipient given its public key. Only the recipient can decrypt these messages,
#' using its private key.
#'
#' While the recipient can verify the integrity of the message, it cannot verify the
#' identity of the sender. For sending authenticated encrypted messages, use
#' \link{auth_encrypt} and \link{auth_decrypt}.
#'
#' @export
#' @rdname simple
#' @name Simple encryption
#' @useDynLib sodium R_seal_box
#' @references \url{https://doc.libsodium.org/public-key_cryptography/sealed_boxes.html}
#' @param msg message to be encrypted
#' @param key private key of the receiver
#' @param pubkey public key of the receiver
#' @param bin encrypted ciphertext
#' @examples # Generate keypair
#' key <- keygen()
#' pub <- pubkey(key)
#'
#' # Encrypt message with pubkey
#' msg <- serialize(iris, NULL)
#' ciphertext <- simple_encrypt(msg, pub)
#'
#' # Decrypt message with private key
#' out <- simple_decrypt(ciphertext, key)
#' stopifnot(identical(out, msg))
simple_encrypt <- function(msg, pubkey){
  stopifnot(is.raw(msg))
  stopifnot(is.raw(pubkey))
  .Call(R_seal_box, msg, pubkey)
}

#' @export
#' @rdname simple
#' @useDynLib sodium R_seal_open
simple_decrypt <- function(bin, key){
  stopifnot(is.raw(bin))
  stopifnot(is.raw(key))
  .Call(R_seal_open, bin, key)
}

if(!file.exists("../windows/sodium/include/sodium.h")){
  unlink("../windows", recursive = TRUE)
  url <- if(grepl("aarch", R.version$platform)){
    "https://github.com/r-windows/bundles/releases/download/libsodium-1.0.18/libsodium-1.0.18-clang-aarch64.tar.xz"
  } else if(grepl("clang", Sys.getenv('R_COMPILED_BY'))){
    "https://github.com/r-windows/bundles/releases/download/libsodium-1.0.18/libsodium-1.0.18-clang-x86_64.tar.xz"
  }  else if(getRversion() >= "4.2") {
    "https://github.com/r-windows/bundles/releases/download/libsodium-1.0.18/libsodium-1.0.18-ucrt-x86_64.tar.xz"
  } else {
    "https://github.com/rwinlib/sodium/archive/v1.0.12.tar.gz"
  }
  download.file(url, basename(url), quiet = TRUE)
  dir.create("../windows", showWarnings = FALSE)
  untar(basename(url), exdir = "../windows", tar = 'internal')
  unlink(basename(url))
  setwd("../windows")
  file.rename(list.files(), 'sodium')
}

#' Password Storage
#'
#' Wrapper that implements best practices for storing passwords based on scrypt with
#' a random salt.
#'
#' The \link{password_store} function returns an ASCII encoded string which contains
#' the result of a memory-hard, CPU-intensive hash function along with the automatically
#' generated salt and other parameters required to verify the password. Use
#' \link{password_verify} to verify a password from this string.
#'
#' @export
#' @rdname password
#' @name Password storage
#' @aliases password
#' @param password a string of length one with a password
#' @param hash a hash string of length one generated by \code{password_store}
#' @useDynLib sodium R_password_hash
#' @references \url{https://doc.libsodium.org/password_hashing/}
#' @examples # Example password
#' password <- "I like cookies"
#'
#' # Hash is what you store in the database
#' hash <- password_store(password)
#'
#' # To verify the password when the user logs in
#' stopifnot(password_verify(hash, password))
password_store <- function(password){
  .Call(R_password_hash, password)
}

#' @export
#' @rdname password
#' @useDynLib sodium R_password_verify
password_verify <- function(hash, password){
  .Call(R_password_verify, hash, password)
}

---
title: "How does cryptography work?"
output: html_document
date: "`r Sys.Date()`"
vignette: >
  %\VignetteIndexEntry{How does cryptography work}
  %\VignetteEngine{knitr::rmarkdown}
  \usepackage[utf8]{inputenc}
---

```{r, echo = FALSE, message = FALSE}
knitr::opts_chunk$set(comment = "")
library(sodium)

# hack for printable bits 
random <- function(n = 1){
  if(n != nchar("TTIP is evil"))
    return(sodium::random(n))
  repeat {
    x <- sodium::random(n)
    y <- base::xor(charToRaw("TTIP is evil"), x)
    if(all(c(x,y) != 0)) return(x) 
  }
}
```

This page attempts to give a very basic conceptual introduction to cryptographic methods. Before we start the usual disclaimer:

___I am not a cryptographer. This document is only for educational purposes. Crypto is hard, you should never trust your home-grown implementation. Unless you're a cryptographer you will probably overlook some crucial details. Developers should only use the high-level functions that have been implemented by an actual cryptographer.___

Now that we got this is out of the way, let's start hacking :)

### The XOR operator 

The bitwise [XOR operator](https://en.wikipedia.org/wiki/Exclusive_or#Truth_table) outputs `true` only when both inputs differ (one is `true`, the other is `false`). It is sometimes called an *invertor* because the output of a bit in `x` gets inverted if and only if the corresponding bit in `y` is true:

```{r}
# XOR two (8bit) bytes 'x' and 'y'
x <- as.raw(0x7a)
y <- as.raw(0xe4)
z <- base::xor(x, y)
dput(z)

# Show the bits in each byte
cbind(x = rawToBits(x), y = rawToBits(y), z = rawToBits(z))
```

In cryptography we `xor` a message `x` with secret random data `y`. Because each bit in `y` is randomly `true` with probability 0.5, the `xor` output is completely random and uncorrelated to `x`. This is called *perfect secrecy*. Only if we know `y` we can decipher the message `x`.

```{r}
# Encrypt message using random one-time-pad
msg <- charToRaw("TTIP is evil")
one_time_pad <- random(length(msg))
ciphertext <- base::xor(msg, one_time_pad)

# It's really encrypted
rawToChar(ciphertext)

# Decrypt with same pad
rawToChar(base::xor(ciphertext, one_time_pad))
```

This method is perfectly secure and forms the basis for most cryptograhpic methods. However the challenge is generating and communicating unique pseudo-random `y` data every time we want to encrypt something. One-time-pads as in the example are not very practical for large messages. Also we should never re-use a one-time-pad `y` for encrypting multiple messages, as this  compromises the secrecy. 

### Stream ciphers

The solution to this problem are stream ciphers. A *stream cipher* generates a unique stream of pseudo-random data based on a secret `key` and a unique `nonce`. For a given set of parameters the stream cipher always generates the same stream of data. Sodium implements a few popular stream ciphers:

```{r}
password <- "My secret passphrase"
key <- hash(charToRaw(password))
nonce <- random(8)
chacha20(size = 20, key, nonce)
```

Each stream requires a `key` and a `nonce`. The key forms the shared secret and should only be known to trusted parties. The `nonce` is not secret and is stored or sent along with the ciphertext. The purpose of the `nonce` is to make a random stream unique to protect gainst re-use attacks. This way you can re-use a your key to encrypt multiple messages, as long as you never re-use the same nonce.

```{r}
salsa20(size = 20, key, nonce)
```

Over the years cryptographers have come up with many more variants. Many stream ciphers are based on a block cipher such as [AES](https://en.wikipedia.org/wiki/Advanced_Encryption_Standard): a keyed permutation of fixed length amount of data. The block ciphers get chained in a particular [mode of operation](https://en.wikipedia.org/wiki/Block_cipher_mode_of_operation) which repeatedly applies the cipher's single-block operation to securely transform amounts of data larger than a block.

We are not going to discuss implementation details, but you could probably come up with something yourself. For example you could use a hash function such `sha256` as the block cipher and append counter which is incremented for each block (this is called CTR mode).


```{r}
# Illustrative example.
sha256_ctr <- function(size, key, nonce){
  n <- ceiling(size/32)
  output <- raw()
  for(i in 1:n){
    counter <- packBits(intToBits(i))
    block <- sha256(c(key, nonce, counter))
    output <- c(output, block)
  }
  return(output[1:size])
}
```

This allows us to generate an arbitrary length stream from a single secret key:

```{r}
password <- "My secret passphrase"
key <- hash(charToRaw(password))
nonce <- random(8)
sha256_ctr(50, key, nonce)
```



In practice, you should never write your own ciphers. A lot of [research](http://citeseerx.ist.psu.edu/viewdoc/download?doi=10.1.1.58.1811&rep=rep1&type=pdf) goes into studying the properties of block ciphers under various modes of operation. In the remainder we just use the standard Sodium ciphers: [`chacha20`](https://download.libsodium.org/libsodium/content/advanced/chacha20.html), [`salsa20`](https://download.libsodium.org/libsodium/content/advanced/salsa20.html) or  [`xsalsa20`](https://download.libsodium.org/libsodium/content/advanced/xsalsa20.html).

### Symmetric encryption

Symmetric encryption means that the same secret key is used for both encryption and decryption. All that is needed to implement symmetric encryption is `xor` and a stream cipher. For example to encrypt an arbitrary length `message` using `password`:

```{r}
# Encrypt 'message' using 'password'
myfile <- file.path(R.home(), "COPYING")
message <- readBin(myfile, raw(), file.info(myfile)$size)
passwd <- charToRaw("My secret passphrase")
```

A hash function converts the password to a key of suitable size for the stream cipher, which we use to generate a psuedo random stream of equal length to the message:

```{r}
# Basic secret key encryption
key <- hash(passwd)
nonce8 <- random(8)
stream <- chacha20(length(message), key, nonce8)
ciphertext <- base::xor(stream, message)
```

Now the `ciphertext` is an encrypted version of the message. Only those that know the `key` and the `nonce` can re-generate the same keystream in order to `xor` the ciphertext back into the original message. 

```{r}
# Decrypt with the same key
key <- hash(charToRaw("My secret passphrase"))
stream <- chacha20(length(ciphertext), key, nonce8)
out <- base::xor(ciphertext, stream)

# Print part of the message
cat(substring(rawToChar(out), 1, 120))
```

The Sodium functions `data_encrypt` and `data_decrypt` provide a more elaborate implementation of the above. This is what you should use in practice for secret key encryption.

Symmetric encryption can be used for e.g. encrypting local data. However because the same secret is used for both encryption and decryption, it is impractical for communication with other parties. For exchanging secure messages we need public key encryption.

### Public-key encryption and Diffie-Hellman

Rather than using a single secret-key, assymetric (public key) encryption requires a *keypair*, consisting of a *public key* for encryption and a *private-key* for decryption. Data that is encrypted using a given public key can only be decrypted using the corresponding private key. 

The public key is not confidential and can be shared on e.g. a website or keyserver. This allows anyone to send somebody a secure message by encrypting it with the receivers public key. The encrypted message will only be readable by the owner of the corresponding private key.

```{r}
# Create keypair
key <- keygen()
pub <- pubkey(key)

# Encrypt message for receiver using his/her public key
msg <- serialize(iris, NULL)
ciphertext <- simple_encrypt(msg, pub)

# Receiver decrypts with his/her private key
out <- simple_decrypt(ciphertext, key)
identical(msg, out)
```

How does this work? Public key encryption makes use of [Diffie-Hellman](http://mathworld.wolfram.com/Diffie-HellmanProtocol.html) (D-H): a method which allows two parties that have no prior knowledge of each other to jointly establish a shared secret key over an insecure channel. In the most simple case, both parties generate a temporary keypair and exchange their public key over the insecure channel. Then both parties use the D-H function to calculcate the (same) shared secret key by combining their own private key with the other person's public key:

```{r}
# Bob generates keypair
bob_key <- keygen()
bob_pubkey <- pubkey(bob_key)

# Alice generates keypair
alice_key <- keygen()
alice_pubkey <- pubkey(alice_key)

# After Bob and Alice exchange pubkey they can both derive the secret
alice_secret <- diffie_hellman(alice_key, bob_pubkey)
bob_secret <- diffie_hellman(bob_key, alice_pubkey)
identical(alice_secret, bob_secret)
```

Once the shared secret has been established, both parties can discard their temporary public/private key and use the shared secret to start encrypting communications with symmetric encryption as discussed earlier. Because the shared secret cannot be calculated using only the public keys, the process is safe from eavesdroppers.

The classical [Diffie-Hellman method](http://mathworld.wolfram.com/Diffie-HellmanProtocol.html) is based on the discrete logarithm problem with large prime numbers. Sodium uses [curve25519](http://cr.yp.to/ecdh/curve25519-20060209.pdf), a state-of-the-art D-H function by Daniel Bernsteinan designed for use with the elliptic curve Diffie–Hellman (ECDH) key agreement scheme.

---
title: "How does cryptography work?"
output: html_document
date: "`r Sys.Date()`"
vignette: >
  %\VignetteIndexEntry{How does cryptography work}
  %\VignetteEngine{knitr::rmarkdown}
  \usepackage[utf8]{inputenc}
---

```{r, echo = FALSE, message = FALSE}
knitr::opts_chunk$set(comment = "")
library(sodium)

# hack for printable bits 
random <- function(n = 1){
  if(n != nchar("TTIP is evil"))
    return(sodium::random(n))
  repeat {
    x <- sodium::random(n)
    y <- base::xor(charToRaw("TTIP is evil"), x)
    if(all(c(x,y) != 0)) return(x) 
  }
}
```

This page attempts to give a very basic conceptual introduction to cryptographic methods. Before we start the usual disclaimer:

___I am not a cryptographer. This document is only for educational purposes. Crypto is hard, you should never trust your home-grown implementation. Unless you're a cryptographer you will probably overlook some crucial details. Developers should only use the high-level functions that have been implemented by an actual cryptographer.___

Now that we got this is out of the way, let's start hacking :)

### The XOR operator 

The bitwise [XOR operator](https://en.wikipedia.org/wiki/Exclusive_or#Truth_table) outputs `true` only when both inputs differ (one is `true`, the other is `false`). It is sometimes called an *invertor* because the output of a bit in `x` gets inverted if and only if the corresponding bit in `y` is true:

```{r}
# XOR two (8bit) bytes 'x' and 'y'
x <- as.raw(0x7a)
y <- as.raw(0xe4)
z <- base::xor(x, y)
dput(z)

# Show the bits in each byte
cbind(x = rawToBits(x), y = rawToBits(y), z = rawToBits(z))
```

In cryptography we `xor` a message `x` with secret random data `y`. Because each bit in `y` is randomly `true` with probability 0.5, the `xor` output is completely random and uncorrelated to `x`. This is called *perfect secrecy*. Only if we know `y` we can decipher the message `x`.

```{r}
# Encrypt message using random one-time-pad
msg <- charToRaw("TTIP is evil")
one_time_pad <- random(length(msg))
ciphertext <- base::xor(msg, one_time_pad)

# It's really encrypted
rawToChar(ciphertext)

# Decrypt with same pad
rawToChar(base::xor(ciphertext, one_time_pad))
```

This method is perfectly secure and forms the basis for most cryptograhpic methods. However the challenge is generating and communicating unique pseudo-random `y` data every time we want to encrypt something. One-time-pads as in the example are not very practical for large messages. Also we should never re-use a one-time-pad `y` for encrypting multiple messages, as this  compromises the secrecy. 

### Stream ciphers

The solution to this problem are stream ciphers. A *stream cipher* generates a unique stream of pseudo-random data based on a secret `key` and a unique `nonce`. For a given set of parameters the stream cipher always generates the same stream of data. Sodium implements a few popular stream ciphers:

```{r}
password <- "My secret passphrase"
key <- hash(charToRaw(password))
nonce <- random(8)
chacha20(size = 20, key, nonce)
```

Each stream requires a `key` and a `nonce`. The key forms the shared secret and should only be known to trusted parties. The `nonce` is not secret and is stored or sent along with the ciphertext. The purpose of the `nonce` is to make a random stream unique to protect gainst re-use attacks. This way you can re-use a your key to encrypt multiple messages, as long as you never re-use the same nonce.

```{r}
salsa20(size = 20, key, nonce)
```

Over the years cryptographers have come up with many more variants. Many stream ciphers are based on a block cipher such as [AES](https://en.wikipedia.org/wiki/Advanced_Encryption_Standard): a keyed permutation of fixed length amount of data. The block ciphers get chained in a particular [mode of operation](https://en.wikipedia.org/wiki/Block_cipher_mode_of_operation) which repeatedly applies the cipher's single-block operation to securely transform amounts of data larger than a block.

We are not going to discuss implementation details, but you could probably come up with something yourself. For example you could use a hash function such `sha256` as the block cipher and append counter which is incremented for each block (this is called CTR mode).


```{r}
# Illustrative example.
sha256_ctr <- function(size, key, nonce){
  n <- ceiling(size/32)
  output <- raw()
  for(i in 1:n){
    counter <- packBits(intToBits(i))
    block <- sha256(c(key, nonce, counter))
    output <- c(output, block)
  }
  return(output[1:size])
}
```

This allows us to generate an arbitrary length stream from a single secret key:

```{r}
password <- "My secret passphrase"
key <- hash(charToRaw(password))
nonce <- random(8)
sha256_ctr(50, key, nonce)
```



In practice, you should never write your own ciphers. In the remainder we just use the standard Sodium ciphers: [`chacha20`](https://libsodium.gitbook.io/doc/advanced/stream_ciphers/chacha20), [`salsa20`](https://libsodium.gitbook.io/doc/advanced/stream_ciphers/salsa20) or  [`xsalsa20`](https://libsodium.gitbook.io/doc/advanced/stream_ciphers/xsalsa20).

### Symmetric encryption

Symmetric encryption means that the same secret key is used for both encryption and decryption. All that is needed to implement symmetric encryption is `xor` and a stream cipher. For example to encrypt an arbitrary length `message` using `password`:

```{r}
# Encrypt 'message' using 'password'
myfile <- file.path(R.home(), "COPYING")
message <- readBin(myfile, raw(), file.info(myfile)$size)
passwd <- charToRaw("My secret passphrase")
```

A hash function converts the password to a key of suitable size for the stream cipher, which we use to generate a psuedo random stream of equal length to the message:

```{r}
# Basic secret key encryption
key <- hash(passwd)
nonce8 <- random(8)
stream <- chacha20(length(message), key, nonce8)
ciphertext <- base::xor(stream, message)
```

Now the `ciphertext` is an encrypted version of the message. Only those that know the `key` and the `nonce` can re-generate the same keystream in order to `xor` the ciphertext back into the original message. 

```{r}
# Decrypt with the same key
key <- hash(charToRaw("My secret passphrase"))
stream <- chacha20(length(ciphertext), key, nonce8)
out <- base::xor(ciphertext, stream)

# Print part of the message
cat(substring(rawToChar(out), 1, 120))
```

The Sodium functions `data_encrypt` and `data_decrypt` provide a more elaborate implementation of the above. This is what you should use in practice for secret key encryption.

Symmetric encryption can be used for e.g. encrypting local data. However because the same secret is used for both encryption and decryption, it is impractical for communication with other parties. For exchanging secure messages we need public key encryption.

### Public-key encryption and Diffie-Hellman

Rather than using a single secret-key, assymetric (public key) encryption requires a *keypair*, consisting of a *public key* for encryption and a *private-key* for decryption. Data that is encrypted using a given public key can only be decrypted using the corresponding private key. 

The public key is not confidential and can be shared on e.g. a website or keyserver. This allows anyone to send somebody a secure message by encrypting it with the receivers public key. The encrypted message will only be readable by the owner of the corresponding private key.

```{r}
# Create keypair
key <- keygen()
pub <- pubkey(key)

# Encrypt message for receiver using his/her public key
msg <- serialize(iris, NULL)
ciphertext <- simple_encrypt(msg, pub)

# Receiver decrypts with his/her private key
out <- simple_decrypt(ciphertext, key)
identical(msg, out)
```

How does this work? Public key encryption makes use of Diffie-Hellman (D-H): a method which allows two parties that have no prior knowledge of each other to jointly establish a shared secret key over an insecure channel. In the most simple case, both parties generate a temporary keypair and exchange their public key over the insecure channel. Then both parties use the D-H function to calculcate the (same) shared secret key by combining their own private key with the other person's public key:

```{r}
# Bob generates keypair
bob_key <- keygen()
bob_pubkey <- pubkey(bob_key)

# Alice generates keypair
alice_key <- keygen()
alice_pubkey <- pubkey(alice_key)

# After Bob and Alice exchange pubkey they can both derive the secret
alice_secret <- diffie_hellman(alice_key, bob_pubkey)
bob_secret <- diffie_hellman(bob_key, alice_pubkey)
identical(alice_secret, bob_secret)
```

Once the shared secret has been established, both parties can discard their temporary public/private key and use the shared secret to start encrypting communications with symmetric encryption as discussed earlier. Because the shared secret cannot be calculated using only the public keys, the process is safe from eavesdroppers.

The classical Diffie-Hellman method is based on the discrete logarithm problem with large prime numbers. Sodium uses [curve25519](https://cr.yp.to/ecdh/curve25519-20060209.pdf), a state-of-the-art D-H function by Daniel Bernsteinan designed for use with the elliptic curve Diffie–Hellman (ECDH) key agreement scheme.

---
title: "Sodium: A Modern and Easy-to-Use Crypto Library"
output:
  html_document:
    toc: false
date: "`r Sys.Date()`"
vignette: >
  %\VignetteIndexEntry{Introduction to Sodium for R}
  %\VignetteEngine{knitr::rmarkdown}
  \usepackage[utf8]{inputenc}
---

```{r, echo = FALSE, message = FALSE}
knitr::opts_chunk$set(comment = "")
library(sodium)
```

The sodium R package provides bindings to [libsodium](https://libsodium.gitbook.io/): a modern, easy-to-use software library for encryption, decryption, signatures, password hashing and more. 

The goal of Sodium is to provide the core operations needed to build higher-level cryptographic tools. It is not intended for implementing standardized protocols such as TLS, SSH or GPG. Sodium only supports a limited set of state-of-the-art elliptic curve methods, resulting in a simple but very powerful tool-kit for building secure applications.


|                              | Authenticated Encryption         | Encryption only                     | Authentication only        | 
|-----------------------------:|:--------------------------------:|:-----------------------------------:|:--------------------------:|
| __Symmetric__ *(secret key)*:  | [`data_encrypt` / `data_decrypt`](#secret-key-encryption) | | [`data_tag`](#secret-key-authentication) | 
| __Asymmetric__ *(public+private key)*: | [`auth_encrypt` / `auth_decrypt`](#public-key-authenticated-encryption)  | [`simple_encrypt` / `simple_decrypt`](#public-key-encryption) | [`sig_sign` / `sig_verify`](#public-key-authentication-signatures) | 
|                              |                                  |                                     |                            |

### Using Sodium

All Sodium functions operate on binary data, called 'raw' vectors in R. Use `charToRaw` and `rawToChar` to convert between strings and raw vectors. Alternatively `hex2bin` and `bin2hex` can convert between binary data to strings in hex notation:

```{r}
test <- hash(charToRaw("test 123"))
str <- bin2hex(test)
print(str)
hex2bin(str)
```

### Random data generator

The `random()` function generates n bytes of unpredictable data, suitable for creating secret keys.

```{r}
secret <- random(8)
print(secret)
```

Implementation is platform specific, see the [docs](https://libsodium.gitbook.io/doc/generating_random_data) for  details.

### Hash functions

Sodium has several hash functions including `hash()`, `shorthash()`, `sha256()`, `sha512` and `scrypt()`. The generic `hash()` is usually recommended. It uses [blake2b](https://doc.libsodium.org/hashing/generic_hashing.html) with a configurable size between 16 bytes (128bit) and 64 bytes (512bit).

```{r}
# Generate keys from passphrase
passphrase <- charToRaw("This is super secret")
hash(passphrase)
hash(passphrase, size = 16)
hash(passphrase, size = 64)
```

The `shorthash()` function is a special 8 byte (64 bit) hash based on [SipHash-2-4](https://doc.libsodium.org/hashing/short-input_hashing.html). The output of this function is only 64 bits (8 bytes). It is useful for in e.g. Hash tables, but it should not be considered collision-resistant.


### Secret key encryption

Symmetric encryption uses the same secret key for both encryption and decryption. It is mainly useful for encrypting local data, or as a building block for more complex methods.

Most encryption methods require a `nonce`: a piece of non-secret unique data that is used to randomize the cipher. This allows for safely using the same `key` for encrypting multiple messages. The nonce should be stored or shared along with the ciphertext.

```{r}
key <- hash(charToRaw("This is a secret passphrase"))
msg <- serialize(iris, NULL)

# Encrypt with a random nonce
nonce <- random(24)
cipher <- data_encrypt(msg, key, nonce)

# Decrypt with same key and nonce
orig <- data_decrypt(cipher, key, nonce)
identical(iris, unserialize(orig))
```

Because the secret has to be known by all parties, symmetric encryption by itself is often impractical for communication with third parties. For this we need asymmetric (public key) methods.

### Secret key authentication

Secret key authentication is called tagging in Sodium. A tag is basically a hash of the data together with a secret key. 

```{r}
key <- hash(charToRaw("This is a secret passphrase"))
msg <- serialize(iris, NULL)
mytag <- data_tag(msg, key)
```

To verify the integrity of the data at a later point in time, simply re-calculate the tag with the same key:

```{r}
stopifnot(identical(mytag, data_tag(msg, key)))
```

The secret key protects against forgery of the data+tag by an intermediate party, as would be possible with a regular checksum.

### Public key encryption

Where symmetric methods use the same secret key for encryption and decryption, asymmetric methods use a key-pair consisting of a public key and private key. The private key is secret and only known by its owner. The public key on the other hand can be shared with anyone. Public keys are often published on the user's website or posted in public directories or keyservers.

```{r}
key <- keygen()
pub <- pubkey(key)
```

In public key encryption, data encrypted with a public key can only be decrypted using the corresponding private key. This allows anyone to send somebody a secure message by encrypting it with the receivers public key. The encrypted message will only be readable by the owner of the corresponding private key. 

```{r}
# Encrypt message with pubkey
msg <- serialize(iris, NULL)
ciphertext <- simple_encrypt(msg, pub)

# Decrypt message with private key
out <- simple_decrypt(ciphertext, key)
stopifnot(identical(out, msg))
```


### Public key authentication (signatures)

Public key authentication works the other way around. First, the owner of the private key creates a 'signature' (an authenticated checksum) for a message in a way that allows anyone who knows his/her public key to verify the integrity of the message and identity of the sender.

Currently sodium requires a different type of key-pair for signatures (ed25519) than for encryption (curve25519).

```{r}
# Generate signature keypair
key <- sig_keygen()
pubkey <- sig_pubkey(key)

# Create signature with private key
msg <- serialize(iris, NULL)
sig <- sig_sign(msg, key)
print(sig)

# Verify a signature from public key
sig_verify(msg, sig, pubkey)
```

Signatures are useful when the message itself is not confidential but integrity is important. A common use is for software repositories where to include an index file with checksums for all packages, signed by the repository maintainer. This allows client package managers to verify that the binaries were not manipulated by intermediate parties during the distribution process.

### Public key authenticated encryption

Authenticated encryption implements best practices for secure messaging. It requires that both sender and receiver have a keypair and know each other's public key. Each message gets authenticated with the key of the sender and encrypted with the key of the receiver.

```{r}
# Bob's keypair:
bob_key <- keygen()
bob_pubkey <- pubkey(bob_key)

# Alice's keypair:
alice_key <- keygen()
alice_pubkey <- pubkey(alice_key)

# Bob sends encrypted message for Alice:
msg <- charToRaw("TTIP is evil")
ciphertext <- auth_encrypt(msg, bob_key, alice_pubkey)

# Alice verifies and decrypts with her key
out <- auth_decrypt(ciphertext, alice_key, bob_pubkey)
stopifnot(identical(out, msg))

# Alice sends encrypted message for Bob
msg <- charToRaw("Let's protest")
ciphertext <- auth_encrypt(msg, alice_key, bob_pubkey)

# Bob verifies and decrypts with his key
out <- auth_decrypt(ciphertext, bob_key, alice_pubkey)
stopifnot(identical(out, msg))
```

Note that even though public keys are not confidential, you should not exchange them over the same insecure channel you are trying to protect. If the connection is being tampered with, the attacker could simply replace the key with another one to hijack the interaction.

---
title: "Sodium: A Modern and Easy-to-Use Crypto Library"
output:
  html_document:
    toc: false
date: "`r Sys.Date()`"
vignette: >
  %\VignetteIndexEntry{Introduction to Sodium for R}
  %\VignetteEngine{knitr::rmarkdown}
  \usepackage[utf8]{inputenc}
---

```{r, echo = FALSE, message = FALSE}
knitr::opts_chunk$set(comment = "")
library(sodium)
```

The sodium R package provides bindings to [libsodium](https://download.libsodium.org/doc/): a modern, easy-to-use software library for encryption, decryption, signatures, password hashing and more. 

The goal of Sodium is to provide the core operations needed to build higher-level cryptographic tools. It is not intended for implementing standardized protocols such as TLS, SSH or GPG. Sodium only supports a limited set of state-of-the-art elliptic curve methods, resulting in a simple but very powerful tool-kit for building secure applications.


|                              | Authenticated Encryption         | Encryption only                     | Authentication only        | 
|-----------------------------:|:--------------------------------:|:-----------------------------------:|:--------------------------:|
| __Symmetric__ *(secret key)*:  | [`data_encrypt` / `data_decrypt`](#secret-key-encryption) | | [`data_tag`](#secret-key-authentication) | 
| __Asymmetric__ *(public+private key)*: | [`auth_encrypt` / `auth_decrypt`](#public-key-authenticated-encryption)  | [`simple_encrypt` / `simple_decrypt`](#public-key-encryption) | [`sig_sign` / `sig_verify`](#public-key-authentication-signatures) | 
|                              |                                  |                                     |                            |

### Using Sodium

All Sodium functions operate on binary data, called 'raw' vectors in R. Use `charToRaw` and `rawToChar` to convert between strings and raw vectors. Alternatively `hex2bin` and `bin2hex` can convert between binary data to strings in hex notation:

```{r}
test <- hash(charToRaw("test 123"))
str <- bin2hex(test)
print(str)
hex2bin(str)
```

### Random data generator

The `random()` function generates n bytes of unpredictable data, suitable for creating secret keys.

```{r}
secret <- random(8)
print(secret)
```

Implementation is platform specific, see the [docs](https://download.libsodium.org/doc/generating_random_data/index.html) for  details.

### Hash functions

Sodium has several hash functions including `hash()`, `shorthash()`, `sha256()`, `sha512` and `scrypt()`. The generic `hash()` is usually recommended. It uses [blake2b](https://download.libsodium.org/doc/hashing/generic_hashing.html) with a configurable size between 16 bytes (128bit) and 64 bytes (512bit).

```{r}
# Generate keys from passphrase
passphrase <- charToRaw("This is super secret")
hash(passphrase)
hash(passphrase, size = 16)
hash(passphrase, size = 64)
```

The `shorthash()` function is a special 8 byte (64 bit) hash based on [SipHash-2-4](https://download.libsodium.org/doc/hashing/short-input_hashing.html). The output of this function is only 64 bits (8 bytes). It is useful for in e.g. Hash tables, but it should not be considered collision-resistant.


### Secret key encryption

Symmetric encryption uses the same secret key for both encryption and decryption. It is mainly useful for encrypting local data, or as a building block for more complex methods.

Most encryption methods require a `nonce`: a piece of non-secret unique data that is used to randomize the cipher. This allows for safely using the same `key` for encrypting multiple messages. The nonce should be stored or shared along with the ciphertext.

```{r}
key <- hash(charToRaw("This is a secret passphrase"))
msg <- serialize(iris, NULL)

# Encrypt with a random nonce
nonce <- random(24)
cipher <- data_encrypt(msg, key, nonce)

# Decrypt with same key and nonce
orig <- data_decrypt(cipher, key, nonce)
identical(iris, unserialize(orig))
```

Because the secret has to be known by all parties, symmetric encryption by itself is often impractical for communication with third parties. For this we need asymmetric (public key) methods.

### Secret key authentication

Secret key authentication is called tagging in Sodium. A tag is basically a hash of the data together with a secret key. 

```{r}
key <- hash(charToRaw("This is a secret passphrase"))
msg <- serialize(iris, NULL)
mytag <- data_tag(msg, key)
```

To verify the integrity of the data at a later point in time, simply re-calculate the tag with the same key:

```{r}
stopifnot(identical(mytag, data_tag(msg, key)))
```

The secret key protects against forgery of the data+tag by an intermediate party, as would be possible with a regular checksum.

### Public key encryption

Where symmetric methods use the same secret key for encryption and decryption, asymmetric methods use a key-pair consisting of a public key and private key. The private key is secret and only known by its owner. The public key on the other hand can be shared with anyone. Public keys are often published on the user's website or posted in public directories or keyservers.

```{r}
key <- keygen()
pub <- pubkey(key)
```

In public key encryption, data encrypted with a public key can only be decrypted using the corresponding private key. This allows anyone to send somebody a secure message by encrypting it with the receivers public key. The encrypted message will only be readable by the owner of the corresponding private key. 

```{r}
# Encrypt message with pubkey
msg <- serialize(iris, NULL)
ciphertext <- simple_encrypt(msg, pub)

# Decrypt message with private key
out <- simple_decrypt(ciphertext, key)
stopifnot(identical(out, msg))
```


### Public key authentication (signatures)

Public key authentication works the other way around. First, the owner of the private key creates a 'signature' (an authenticated checksum) for a message in a way that allows anyone who knows his/her public key to verify the integrity of the message and identity of the sender.

Currently sodium requires a different type of key-pair for signatures (ed25519) than for encryption (curve25519).

```{r}
# Generate signature keypair
key <- sig_keygen()
pubkey <- sig_pubkey(key)

# Create signature with private key
msg <- serialize(iris, NULL)
sig <- sig_sign(msg, key)
print(sig)

# Verify a signature from public key
sig_verify(msg, sig, pubkey)
```

Signatures are useful when the message itself is not confidential but integrity is important. A common use is for software repositories where to include an index file with checksums for all packages, signed by the repository maintainer. This allows client package managers to verify that the binaries were not manipulated by intermediate parties during the distribution process.

### Public key authenticated encryption

Authenticated encryption implements best practices for secure messaging. It requires that both sender and receiver have a keypair and know each other's public key. Each message gets authenticated with the key of the sender and encrypted with the key of the receiver.

```{r}
# Bob's keypair:
bob_key <- keygen()
bob_pubkey <- pubkey(bob_key)

# Alice's keypair:
alice_key <- keygen()
alice_pubkey <- pubkey(alice_key)

# Bob sends encrypted message for Alice:
msg <- charToRaw("TTIP is evil")
ciphertext <- auth_encrypt(msg, bob_key, alice_pubkey)

# Alice verifies and decrypts with her key
out <- auth_decrypt(ciphertext, alice_key, bob_pubkey)
stopifnot(identical(out, msg))

# Alice sends encrypted message for Bob
msg <- charToRaw("Let's protest")
ciphertext <- auth_encrypt(msg, alice_key, bob_pubkey)

# Bob verifies and decrypts with his key
out <- auth_decrypt(ciphertext, bob_key, alice_pubkey)
stopifnot(identical(out, msg))
```

Note that even though public keys are not confidential, you should not exchange them over the same insecure channel you are trying to protect. If the connection is being tampered with, the attacker could simply replace the key with another one to hijack the interaction.

---
title: "How does cryptography work?"
output: html_document
date: "`r Sys.Date()`"
vignette: >
  %\VignetteIndexEntry{How does cryptography work}
  %\VignetteEngine{knitr::rmarkdown}
  \usepackage[utf8]{inputenc}
---

```{r, echo = FALSE, message = FALSE}
knitr::opts_chunk$set(comment = "")
library(sodium)

# hack for printable bits 
random <- function(n = 1){
  if(n != nchar("TTIP is evil"))
    return(sodium::random(n))
  repeat {
    x <- sodium::random(n)
    y <- base::xor(charToRaw("TTIP is evil"), x)
    if(all(c(x,y) != 0)) return(x) 
  }
}
```

This page attempts to give a very basic conceptual introduction to cryptographic methods. Before we start the usual disclaimer:

___I am not a cryptographer. This document is only for educational purposes. Crypto is hard, you should never trust your home-grown implementation. Unless you're a cryptographer you will probably overlook some crucial details. Developers should only use the high-level functions that have been implemented by an actual cryptographer.___

Now that we got this is out of the way, let's start hacking :)

### The XOR operator 

The bitwise [XOR operator](https://en.wikipedia.org/wiki/Exclusive_or#Truth_table) outputs `true` only when both inputs differ (one is `true`, the other is `false`). It is sometimes called an *invertor* because the output of a bit in `x` gets inverted if and only if the corresponding bit in `y` is true:

```{r}
# XOR two (8bit) bytes 'x' and 'y'
x <- as.raw(0x7a)
y <- as.raw(0xe4)
z <- base::xor(x, y)
dput(z)

# Show the bits in each byte
cbind(x = rawToBits(x), y = rawToBits(y), z = rawToBits(z))
```

In cryptography we `xor` a message `x` with secret random data `y`. Because each bit in `y` is randomly `true` with probability 0.5, the `xor` output is completely random and uncorrelated to `x`. This is called *perfect secrecy*. Only if we know `y` we can decipher the message `x`.

```{r}
# Encrypt message using random one-time-pad
msg <- charToRaw("TTIP is evil")
one_time_pad <- random(length(msg))
ciphertext <- base::xor(msg, one_time_pad)

# It's really encrypted
rawToChar(ciphertext)

# Decrypt with same pad
rawToChar(base::xor(ciphertext, one_time_pad))
```

This method is perfectly secure and forms the basis for most cryptograhpic methods. However the challenge is generating and communicating unique pseudo-random `y` data every time we want to encrypt something. One-time-pads as in the example are not very practical for large messages. Also we should never re-use a one-time-pad `y` for encrypting multiple messages, as this  compromises the secrecy. 

### Stream ciphers

The solution to this problem are stream ciphers. A *stream cipher* generates a unique stream of pseudo-random data based on a secret `key` and a unique `nonce`. For a given set of parameters the stream cipher always generates the same stream of data. Sodium implements a few popular stream ciphers:

```{r}
password <- "My secret passphrase"
key <- hash(charToRaw(password))
nonce <- random(8)
chacha20(size = 20, key, nonce)
```

Each stream requires a `key` and a `nonce`. The key forms the shared secret and should only be known to trusted parties. The `nonce` is not secret and is stored or sent along with the ciphertext. The purpose of the `nonce` is to make a random stream unique to protect gainst re-use attacks. This way you can re-use a your key to encrypt multiple messages, as long as you never re-use the same nonce.

```{r}
salsa20(size = 20, key, nonce)
```

Over the years cryptographers have come up with many more variants. Many stream ciphers are based on a block cipher such as [AES](https://en.wikipedia.org/wiki/Advanced_Encryption_Standard): a keyed permutation of fixed length amount of data. The block ciphers get chained in a particular [mode of operation](https://en.wikipedia.org/wiki/Block_cipher_mode_of_operation) which repeatedly applies the cipher's single-block operation to securely transform amounts of data larger than a block.

We are not going to discuss implementation details, but you could probably come up with something yourself. For example you could use a hash function such `sha256` as the block cipher and append counter which is incremented for each block (this is called CTR mode).


```{r}
# Illustrative example.
sha256_ctr <- function(size, key, nonce){
  n <- ceiling(size/32)
  output <- raw()
  for(i in 1:n){
    counter <- packBits(intToBits(i))
    block <- sha256(c(key, nonce, counter))
    output <- c(output, block)
  }
  return(output[1:size])
}
```

This allows us to generate an arbitrary length stream from a single secret key:

```{r}
password <- "My secret passphrase"
key <- hash(charToRaw(password))
nonce <- random(8)
sha256_ctr(50, key, nonce)
```



In practice, you should never write your own ciphers. A lot of [research](https://citeseerx.ist.psu.edu/viewdoc/download?doi=10.1.1.58.1811&rep=rep1&type=pdf) goes into studying the properties of block ciphers under various modes of operation. In the remainder we just use the standard Sodium ciphers: [`chacha20`](https://libsodium.gitbook.io/doc/advanced/stream_ciphers/chacha20), [`salsa20`](https://libsodium.gitbook.io/doc/advanced/stream_ciphers/salsa20) or  [`xsalsa20`](https://libsodium.gitbook.io/doc/advanced/stream_ciphers/xsalsa20).

### Symmetric encryption

Symmetric encryption means that the same secret key is used for both encryption and decryption. All that is needed to implement symmetric encryption is `xor` and a stream cipher. For example to encrypt an arbitrary length `message` using `password`:

```{r}
# Encrypt 'message' using 'password'
myfile <- file.path(R.home(), "COPYING")
message <- readBin(myfile, raw(), file.info(myfile)$size)
passwd <- charToRaw("My secret passphrase")
```

A hash function converts the password to a key of suitable size for the stream cipher, which we use to generate a psuedo random stream of equal length to the message:

```{r}
# Basic secret key encryption
key <- hash(passwd)
nonce8 <- random(8)
stream <- chacha20(length(message), key, nonce8)
ciphertext <- base::xor(stream, message)
```

Now the `ciphertext` is an encrypted version of the message. Only those that know the `key` and the `nonce` can re-generate the same keystream in order to `xor` the ciphertext back into the original message. 

```{r}
# Decrypt with the same key
key <- hash(charToRaw("My secret passphrase"))
stream <- chacha20(length(ciphertext), key, nonce8)
out <- base::xor(ciphertext, stream)

# Print part of the message
cat(substring(rawToChar(out), 1, 120))
```

The Sodium functions `data_encrypt` and `data_decrypt` provide a more elaborate implementation of the above. This is what you should use in practice for secret key encryption.

Symmetric encryption can be used for e.g. encrypting local data. However because the same secret is used for both encryption and decryption, it is impractical for communication with other parties. For exchanging secure messages we need public key encryption.

### Public-key encryption and Diffie-Hellman

Rather than using a single secret-key, assymetric (public key) encryption requires a *keypair*, consisting of a *public key* for encryption and a *private-key* for decryption. Data that is encrypted using a given public key can only be decrypted using the corresponding private key. 

The public key is not confidential and can be shared on e.g. a website or keyserver. This allows anyone to send somebody a secure message by encrypting it with the receivers public key. The encrypted message will only be readable by the owner of the corresponding private key.

```{r}
# Create keypair
key <- keygen()
pub <- pubkey(key)

# Encrypt message for receiver using his/her public key
msg <- serialize(iris, NULL)
ciphertext <- simple_encrypt(msg, pub)

# Receiver decrypts with his/her private key
out <- simple_decrypt(ciphertext, key)
identical(msg, out)
```

How does this work? Public key encryption makes use of Diffie-Hellman (D-H): a method which allows two parties that have no prior knowledge of each other to jointly establish a shared secret key over an insecure channel. In the most simple case, both parties generate a temporary keypair and exchange their public key over the insecure channel. Then both parties use the D-H function to calculcate the (same) shared secret key by combining their own private key with the other person's public key:

```{r}
# Bob generates keypair
bob_key <- keygen()
bob_pubkey <- pubkey(bob_key)

# Alice generates keypair
alice_key <- keygen()
alice_pubkey <- pubkey(alice_key)

# After Bob and Alice exchange pubkey they can both derive the secret
alice_secret <- diffie_hellman(alice_key, bob_pubkey)
bob_secret <- diffie_hellman(bob_key, alice_pubkey)
identical(alice_secret, bob_secret)
```

Once the shared secret has been established, both parties can discard their temporary public/private key and use the shared secret to start encrypting communications with symmetric encryption as discussed earlier. Because the shared secret cannot be calculated using only the public keys, the process is safe from eavesdroppers.

The classical Diffie-Hellman method is based on the discrete logarithm problem with large prime numbers. Sodium uses [curve25519](https://cr.yp.to/ecdh/curve25519-20060209.pdf), a state-of-the-art D-H function by Daniel Bernsteinan designed for use with the elliptic curve Diffie–Hellman (ECDH) key agreement scheme.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data-encrypt.R
\name{Symmetric encryption}
\alias{Symmetric encryption}
\alias{data_encrypt}
\alias{data_decrypt}
\alias{data_tag}
\title{Symmetric Encryption and Tagging}
\usage{
data_encrypt(msg, key, nonce = random(24))

data_decrypt(bin, key, nonce = attr(bin, "nonce"))

data_tag(msg, key)
}
\arguments{
\item{msg}{message to be encrypted}

\item{key}{shared secret key used for both encryption and decryption}

\item{nonce}{non-secret unique data to randomize the cipher}

\item{bin}{encrypted ciphertext}
}
\description{
Encryption with authentication using a 256 bit shared secret. Mainly useful for
encrypting local data. For secure communication use public-key encryption
(\link{simple_encrypt} and \link{auth_encrypt}).
}
\details{
Symmetric encryption uses a secret key to encode and decode a message. This can be
used to encrypt local data on disk, or as a building block for more complex methods.

Because the same \code{secret} is used for both encryption and decryption, symmetric
encryption by itself is impractical for communication. For exchanging secure messages
with other parties, use assymetric (public-key) methods (see \link{simple_encrypt} or
\link{auth_encrypt}).

The \code{nonce} is not confidential but required for decryption, and should be
stored or sent along with the ciphertext. The purpose of the \code{nonce} is to
randomize the cipher to protect gainst re-use attacks. This way you can use one
and the same secret for encrypting multiple messages.

The \link{data_tag} function generates an authenticated hash that can be stored
alongside the data to be able to verify the integrity of the data later on. For
public key signatures see \code{sig_sign} instead.
}
\examples{
# 256-bit key
key <- sha256(charToRaw("This is a secret passphrase"))
msg <- serialize(iris, NULL)

# Encrypts with random nonce
cipher <- data_encrypt(msg, key)
orig <- data_decrypt(cipher, key)
stopifnot(identical(msg, orig))

# Tag the message with your key (HMAC)
tag <- data_tag(msg, key)
}
\references{
\url{https://libsodium.gitbook.io/doc/public-key_cryptography/authenticated_encryption}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stream.R
\name{Stream ciphers}
\alias{Stream ciphers}
\alias{chacha20}
\alias{stream}
\alias{xchacha20}
\alias{salsa20}
\alias{xsalsa20}
\title{Stream Ciphers}
\usage{
chacha20(size, key, nonce)

xchacha20(size, key, nonce)

salsa20(size, key, nonce)

xsalsa20(size, key, nonce)
}
\arguments{
\item{size}{length of cipher stream in bytes}

\item{key}{secret key used by the cipher}

\item{nonce}{non-secret unique data to randomize the cipher}
}
\description{
Generate deterministic streams of random data based off a secret key and random nonce.
}
\details{
You usually don't need to call these methods directly. For local encryption
use \link{data_encrypt}. For secure communication use \link{simple_encrypt} or
\link{auth_encrypt}.

Random streams form the basis for most cryptographic methods. Based a shared secret
(the key) we generate a predictable random data stream of equal length as the message
we need to encrypt. Then we \link{xor} the message data with this random stream,
which effectively inverts each byte in the message with probabiliy 0.5. The message
can be decrypted by re-generating exactly the same random data stream and \link{xor}'ing
it back. See the examples.

Each stream generator requires a \code{key} and a \code{nonce}. Both are required to re-generate
the same stream for decryption. The key forms the shared secret and should only known to
the trusted parties. The \code{nonce} is not secret and should be stored or sent along
with the ciphertext. The purpose of the \code{nonce} is to make a random stream unique
to protect gainst re-use attacks. This way you can re-use a your key to encrypt multiple
messages, as long as you never re-use the same nonce.
}
\examples{
# Very basic encryption
myfile <- file.path(R.home(), "COPYING")
message <- readBin(myfile, raw(), file.info(myfile)$size)
passwd <- charToRaw("My secret passphrase")

# Encrypt:
key <- hash(passwd)
nonce8 <- random(8)
stream <- chacha20(length(message), key, nonce8)
ciphertext <- base::xor(stream, message)

# Decrypt:
stream <- chacha20(length(ciphertext), key, nonce8)
out <- base::xor(ciphertext, stream)
stopifnot(identical(out, message))

# Other stream ciphers
stream <- salsa20(10000, key, nonce8)
stream <- xsalsa20(10000, key, random(24))
stream <- xchacha20(10000, key, random(24))

}
\references{
\url{https://libsodium.gitbook.io/doc/advanced/stream_ciphers/xsalsa20}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simple-encrypt.R
\name{Simple encryption}
\alias{Simple encryption}
\alias{simple_encrypt}
\alias{simple_decrypt}
\title{Anonymous Public-key Encryption (Sealed Box)}
\usage{
simple_encrypt(msg, pubkey)

simple_decrypt(bin, key)
}
\arguments{
\item{msg}{message to be encrypted}

\item{pubkey}{public key of the receiver}

\item{bin}{encrypted ciphertext}

\item{key}{private key of the receiver}
}
\description{
Create an encrypted message (sealed box) from a curve25519 public key.
}
\details{
Simple public key encryption allows for sending anonymous encrypted messages to
a recipient given its public key. Only the recipient can decrypt these messages,
using its private key.

While the recipient can verify the integrity of the message, it cannot verify the
identity of the sender. For sending authenticated encrypted messages, use
\link{auth_encrypt} and \link{auth_decrypt}.
}
\examples{
# Generate keypair
key <- keygen()
pub <- pubkey(key)

# Encrypt message with pubkey
msg <- serialize(iris, NULL)
ciphertext <- simple_encrypt(msg, pub)

# Decrypt message with private key
out <- simple_decrypt(ciphertext, key)
stopifnot(identical(out, msg))
}
\references{
\url{https://doc.libsodium.org/public-key_cryptography/sealed_boxes.html}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diffie-hellman.R
\name{Diffie-Hellman}
\alias{Diffie-Hellman}
\alias{diffie_hellman}
\alias{diffie}
\title{Diffie-Hellman}
\usage{
diffie_hellman(key, pubkey)
}
\arguments{
\item{key}{your private key}

\item{pubkey}{other person's public key}
}
\value{
Returns a shared secret key which can be used in e.g. \link{data_encrypt}.
}
\description{
The Diffie-Hellman key exchange method allows two parties that have no prior knowledge
of each other to jointly establish a shared secret key over an insecure channel. This
key can then be used to encrypt subsequent communications using a symmetric key cipher.
}
\details{
Encryption methods as implemented in \link{data_encrypt} require that parties have a
shared secret key. But often we wish to establish a secure channel with a party we have
no prior relationship with. Diffie-hellman is a method for jointly agreeing on a shared
secret without ever exchanging the secret itself. Sodium implements
\href{https://en.wikipedia.org/wiki/Curve25519}{Curve25519}, a state-of-the-art Diffie-Hellman
function suitable for a wide variety of applications.

The method conists of two steps (see examples). First, both parties generate a random private
key and derive the corresponding public key using \link{pubkey}. These public keys are not
confidential and can be exchanged over an insecure channel. After the public keys are exchanged,
both parties will be able to calculate the (same) shared secret by combining his/her own private
key with the other person's public key using \link{diffie_hellman}.

After the shared secret has been established, the private and public keys are disposed,
and parties can start encrypting communications based on the shared secret using e.g.
\link{data_encrypt}. Because the shared secret cannot be calculated using only the public
keys, the process is safe from eavesdroppers.
}
\examples{
# Bob generates keypair
bob_key <- keygen()
bob_pubkey <- pubkey(bob_key)

# Alice generates keypair
alice_key <- keygen()
alice_pubkey <- pubkey(alice_key)

# After Bob and Alice exchange pubkey they can both derive the secret
alice_secret <- diffie_hellman(alice_key, bob_pubkey)
bob_secret <- diffie_hellman(bob_key, alice_pubkey)
stopifnot(identical(alice_secret, bob_secret))
}
\references{
\url{https://doc.libsodium.org/advanced/scalar_multiplication.html}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/password.R
\name{Password storage}
\alias{Password storage}
\alias{password_store}
\alias{password}
\alias{password_verify}
\title{Password Storage}
\usage{
password_store(password)

password_verify(hash, password)
}
\arguments{
\item{password}{a string of length one with a password}

\item{hash}{a hash string of length one generated by \code{password_store}}
}
\description{
Wrapper that implements best practices for storing passwords based on scrypt with
a random salt.
}
\details{
The \link{password_store} function returns an ASCII encoded string which contains
the result of a memory-hard, CPU-intensive hash function along with the automatically
generated salt and other parameters required to verify the password. Use
\link{password_verify} to verify a password from this string.
}
\examples{
# Example password
password <- "I like cookies"

# Hash is what you store in the database
hash <- password_store(password)

# To verify the password when the user logs in
stopifnot(password_verify(hash, password))
}
\references{
\url{https://doc.libsodium.org/password_hashing/}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utilities.R
\name{Sodium utilities}
\alias{Sodium utilities}
\alias{bin2hex}
\alias{helpers}
\alias{hex2bin}
\alias{random}
\title{Sodium Utilities}
\usage{
bin2hex(bin)

hex2bin(hex, ignore = ":")

random(n = 1)
}
\arguments{
\item{bin}{raw vector with binary data to convert to hex string}

\item{hex}{a string with hexadecimal characters to parse into a binary (raw) vector.}

\item{ignore}{a string with characters to ignore from \code{hex}. See example.}

\item{n}{number of random bytes or numbers to generate}
}
\description{
The functions \code{bin2hex} and \code{hex2bin} convert between binary (raw)
vectors and corresponding string in hexadecimal notation. The \code{random}
function generates \code{n} crypto secure random bytes.
}
\examples{
# Convert raw to hex string and back
test <- charToRaw("test 123")
x <- bin2hex(test)
y <- hex2bin(x)
stopifnot(identical(test, y))
stopifnot(identical(x, paste(test, collapse = "")))

# Parse text with characters
x2 <- paste(test, collapse = ":")
y2 <- hex2bin(x2, ignore = ":")
stopifnot(identical(test, y2))
}
\concept{sodium}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/keygen.R
\name{Key generation}
\alias{Key generation}
\alias{keygen}
\alias{pubkey}
\title{Keypair Generation}
\usage{
keygen(seed = random(32))

pubkey(key)
}
\arguments{
\item{seed}{random data to seed the keygen}

\item{key}{private key for which to calculate the public key}
}
\description{
Functions to generate a random private key and calculate the corresponding curve25519
public key.
}
\details{
Asymmetric methods rely on public-private keypairs. The private keys are secret and
should never be shared with anyone. The public key on the other hand is not confidential
and should be shared with the other parties. Public keys are typically published on the
users's website or posted in public directories or keyservers.

The two main applications for public key cryptography are encryption and authentication.

In public key encryption, data that is encrypted using a public key can only be
decrypted using the corresponding private key. This allows anyone to send somebody a
secure message by encrypting it with the receivers public key. The encrypted message
will only be readable by the owner of the corresponding private key. Basic encryption
is implemented in \link{simple_encrypt}.

Authentication works the other way around. In public key authentication, the owner of the
private key creates a 'signature' (an authenticated checksum) for a message in a way that
allows anyone who knows the user's public key to verify that this message was indeed signed
by the owner of the private key.

If both sender and receiver know each other's public key, the two methods can be combined
so that each message going back and forth is signed by the sender and encrypted for the
receiver. This protects both against eavesdropping and MITM tampering, creating a fully
secure channel.
}
\examples{
# Create keypair
key <- keygen()
pub <- pubkey(key)

# Basic encryption
msg <- serialize(iris, NULL)
ciphertext <- simple_encrypt(msg, pub)
out <- simple_decrypt(ciphertext, key)
stopifnot(identical(msg, out))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/auth-encrypt.R
\name{Authenticated encryption}
\alias{Authenticated encryption}
\alias{auth_encrypt}
\alias{auth_decrypt}
\title{Authenticated Encryption}
\usage{
auth_encrypt(msg, key, pubkey, nonce = random(24))

auth_decrypt(bin, key, pubkey, nonce = attr(bin, "nonce"))
}
\arguments{
\item{msg}{message to be encrypted}

\item{key}{your own private key}

\item{pubkey}{other person's public key}

\item{nonce}{non-secret unique data to randomize the cipher}

\item{bin}{encrypted ciphertext generated by \code{secure_send}}
}
\description{
Exchange secure messages through curve25519 authenticated encryption.
}
\details{
Authenticated encryption implements best practices for secure messaging.
It requires that both sender and receiver have a keypair and know each
other's public key. Each message gets authenticated with the key of the
sender and encrypted with the key of the receiver.

Even though public keys are not confidential, you should not exchange them
over the same insecure channel you are trying to protect. If the connection
is being tampered with, the attacker could simply replace the key with another
one to hijack the interaction.

Most people share their public key by posting them on their website or on a
public keyserver. Another alternative is having your public key signed by a
mutually trusted third party. HTTPS does this using Certificate Authorities.
}
\examples{
# Bob's keypair:
bob_key <- keygen()
bob_pubkey <- pubkey(bob_key)

# Alice's keypair:
alice_key <- keygen()
alice_pubkey <- pubkey(alice_key)

# Bob sends encrypted message for Alice:
msg <- charToRaw("TTIP is evil")
ciphertext <- auth_encrypt(msg, bob_key, alice_pubkey)

# Alice verifies and decrypts with her key
out <- auth_decrypt(ciphertext, alice_key, bob_pubkey)
stopifnot(identical(out, msg))

# Alice sends encrypted message for Bob
msg <- charToRaw("Let's protest")
ciphertext <- auth_encrypt(msg, alice_key, bob_pubkey)

# Bob verifies and decrypts with his key
out <- auth_decrypt(ciphertext, bob_key, alice_pubkey)
stopifnot(identical(out, msg))
}
\references{
\url{https://libsodium.gitbook.io/doc/public-key_cryptography/authenticated_encryption}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/signatures.R
\name{Signatures}
\alias{Signatures}
\alias{sig_sign}
\alias{sig}
\alias{sig_verify}
\alias{sig_keygen}
\alias{sig_pubkey}
\title{Create and Verify Signatures}
\usage{
sig_sign(msg, key)

sig_verify(msg, sig, pubkey)

sig_keygen(seed = random(32))

sig_pubkey(key)
}
\arguments{
\item{msg}{message to sign}

\item{key}{private key to sign message with}

\item{sig}{a signature generated by \code{signature_sign}}

\item{pubkey}{a public key of the keypair used by the signature}

\item{seed}{random data to seed the keygen}
}
\description{
Cryptographic signatures can be used to verify the integrity of a message using
the author's public key.
}
\details{
A signature is an authenticated checksum that can be used to check that a message
(any data) was created by a particular author and was not tampered with. The signature is
created using a private key and can be verified from the corresponding public key.

Signatures are used when the message itself is not confidential but integrity is
important. A common use is for software repositories where maintainers include
a signature of the package index. This allows client package managers to verify
that the binaries were not modified by intermediate parties in the distribution
process.

For confidential data, use authenticated encryption (\link{auth_encrypt})
which allows for sending signed and encrypted messages in a single method.

Currently sodium requires a different type of key pairfor signatures  (ed25519)
than for encryption (curve25519).
}
\examples{
# Generate keypair
key <- sig_keygen()
pubkey <- sig_pubkey(key)

# Create signature
msg <- serialize(iris, NULL)
sig <- sig_sign(msg, key)
sig_verify(msg, sig, pubkey)
}
\references{
\url{https://doc.libsodium.org/public-key_cryptography/public-key_signatures.html}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hashing.R
\name{Hash functions}
\alias{Hash functions}
\alias{hash}
\alias{hashing}
\alias{scrypt}
\alias{argon2}
\alias{shorthash}
\alias{sha512}
\alias{sha256}
\title{Hash Functions}
\usage{
hash(buf, key = NULL, size = 32)

scrypt(buf, salt = raw(32), size = 32)

argon2(buf, salt = raw(16), size = 32)

shorthash(buf, key)

sha512(buf, key = NULL)

sha256(buf, key = NULL)
}
\arguments{
\item{buf}{data to be hashed}

\item{key}{key for HMAC hashing. Optional, except for in \code{shorthash}.}

\item{size}{length of the output hash. Must be between 16 and 64 (recommended is 32)}

\item{salt}{non-confidential random data to seed the algorithm}
}
\description{
Functions to calculate cryptographic hash of a message, with optionally a key for
HMAC applications. For storing passwords, use \link{password_store} instead.
}
\details{
The generic \code{hash} function is recommended for most applications. It uses
dynamic length
\href{https://libsodium.gitbook.io/doc/hashing/generic_hashing}{BLAKE2b}
where output size can be any value between 16 bytes (128bit) and 64 bytes (512bit).

The \link{scrypt} hash function is designed to be CPU and memory expensive to protect
against brute force attacks. This algorithm is also used by the \link{password_store}
function.

The \link{argon2} hash function is also designed to be CPU and memory expensive to protect
against brute force attacks. Argon2 is a password-hashing function that summarizes the
state of the art in the design of memory-hard functions

The \code{shorthash} function is a special 8 byte (64 bit) hash based on
\href{https://libsodium.gitbook.io/doc/hashing/short-input_hashing}{SipHash-2-4}.
The output of this function is only 64 bits (8 bytes). It is useful for in e.g.
Hash tables, but it should not be considered collision-resistant.

Hash functions can be used for HMAC by specifying a secret \code{key}. They key size
for \code{shorthash} is 16 bytes, for \code{sha256} it is 32 bytes and for \code{sha512}
it is 64 bytes. For \code{hash} the key size can be any value between 16 and 62,
recommended is at least 32.
}
\examples{
# Basic hashing
msg <- serialize(iris, NULL)
hash(msg)
sha256(msg)
sha512(msg)
scrypt(msg)

# Generate keys from passphrase
passphrase <- charToRaw("This is super secret")
key <- hash(passphrase)
shortkey <- hash(passphrase, size = 16)
longkey <- hash(passphrase, size = 64)

# HMAC (hashing with key)
hash(msg, key = key)
shorthash(msg, shortkey)
sha256(msg, key = key)
sha512(msg, key = longkey)
}
\references{
\url{https://libsodium.gitbook.io/doc/hashing/generic_hashing}
}
